#include "common.h"
#include "cushome.h"

CANVAS canvas;		// \obt@


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//  LoadExecVSH
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int LoadExecVSH(int apitype, char *path)
{
	struct SceKernelLoadExecVSHParam param;

	memset(&param, 0, sizeof(param));
	param.size = sizeof(param);

	switch(apitype)
	{
		case 0x141: //homebrew mode
		case 0x152: //homebrew PSPGo
			param.args = strlen(path)+1;
			param.argp = path;
			param.key = "game";
			break;
		case 0x120: //ISO mode
		case 0x123:
		case 0x125: //ISO PSPGo
			param.args = strlen(EBOOTBIN)+1;
			param.argp = EBOOTBIN;
			param.key = "umdemu";
			break;
		case 0x144: // POPS mode
		case 0x155: // POPS PSPGo
			param.args = strlen(path)+1;
			param.argp = path;
			param.key = "pops";
			break;
	}

	switch(apitype)
	{
		case 0x141: //homebrew mode
			sceKernelLoadExecVSHMs2(path, &param);
			break;
		case 0x152: //homebrew PSPGo
			sceKernelLoadExecVSHEf2(path, &param);
			break;
		case 0x120: //ISO mode
			sceKernelLoadExecVSHDisc(EBOOTBIN, &param);
			break;
		case 0x123:
			sceKernelLoadExecVSHUmdEmuMs1(path, &param);
			break;
		case 0x125: //ISO PSPGo
			sceKernelLoadExecVSHUmdEmuEf1(path, &param);
			break;
		case 0x144: // POPS mode
			sceKernelLoadExecVSHMs5(path, &param);
			break;
		case 0x155: // POPS PSPGo
			sceKernelLoadExecVSHEf5(path, &param);
			break;
		default:
			return -1;
	}
	return 0;
}


u32 GetSetBaseAddrNidLoadExec(int type)
{
	u32 devkit = sceKernelDevkitVersion();

	if(devkit >= PSP_FIRMWARE(0x620))
	{
		if(devkit >= PSP_FIRMWARE(0x660))//6.60-
		{
			switch(type)
			{
				case 0://sceKernelLoadExecVSHEf2
					return 0x032A7938;
				case 1://sceKernelLoadExecVSHUmdEmuMs1
					return 0xF9CFCF2F;
				case 2://sceKernelLoadExecVSHUmdEmuEf1
					return 0xE704ECC3;
				case 3://sceKernelLoadExecVSHMs5
					return 0x7CABED9B;
				case 4://sceKernelLoadExecVSHEf5
					return 0xC7C83B1E;
			}
		}
		else if(devkit >= PSP_FIRMWARE(0x630))//6.30-6.39
		{
			switch(type)
			{
				case 0://sceKernelLoadExecVSHEf2
					return 0xD35D6403;
				case 1://sceKernelLoadExecVSHUmdEmuMs1
					return 0x4A9446E7;
				case 2://sceKernelLoadExecVSHUmdEmuEf1
					return 0x9828D1D9;
				case 3://sceKernelLoadExecVSHMs5
					return 0x6C00E642;
				case 4://sceKernelLoadExecVSHEf5
					return 0x78912B54;
			}
		} else {//6.20
			switch(type)
			{
				case 0://sceKernelLoadExecVSHEf2
					return 0x88F03BAF;
				case 1://sceKernelLoadExecVSHUmdEmuMs1
					return 0x9D1ED0DF;
				case 2://sceKernelLoadExecVSHUmdEmuEf1
					return 0x3FB452EA;
				case 3://sceKernelLoadExecVSHMs5
					return 0xD1DFD7A6;
				case 4://sceKernelLoadExecVSHEf5
					return 0xAA8A929C;
			}
		}
	}
	return 0;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//  StrReplace
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

//http://katsura-kotonoha.sakura.ne.jp/prog/c/tip0000e.shtml
//u@@p
//*********************************************************
//  String  nShift ړB
// ړւ̃|C^ԂB
//*********************************************************
char *StrShift( char *String, size_t nShift )
{
	char *start = String;
	char *stop  = String + strlen( String );
	memmove( start + nShift, start, stop-start+1 );

	return String + nShift;
}//StrShift

//*********************************************************
//  String ̕ From 𕶎 To ŒuB
// u̕ String ̃TCY String ̋L̈𒴂ꍇ͖̓`B
//*********************************************************
char *StrReplace( char *String, const char *From, const char *To )
{
	int   nToLen;   // u镶̒
	int   nFromLen; // 镶̒
	int   nShift;
	char *start;    // Jnʒu
	char *stop;     //  String ̏I[
	char *p;

	nToLen   = strlen( To );
	nFromLen = strlen( From );
	nShift   = nToLen - nFromLen;
	start    = String;
	stop     = String + strlen( String );

	//  String ̐擪當 From 
	while( NULL != ( p = strstr( start, From ) ) )
	{
		//  To ʂł悤ɂ
		start = StrShift( p + nFromLen, nShift );
		stop  = stop + nShift;

		//  To 𕡎
		memmove( p, To, nToLen );
	}

	return String;
}//StrReplace

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//  libmExecTCmd
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

//vOCȊȎXbh~EĊJ
int libmExecTCmd( int cmd )
{
	static int thid_status = LIBM_TRESUME;
	static bool thid_IsDump = false;
	static bool this_IsBusy = false;
	
	static int thid_list_first[MAX_NUMBER_OF_THREADS];
	static int thid_count_first;
	
	static int thid_list_Current[MAX_NUMBER_OF_THREADS];
	static int thid_count_Current;
	
	if( cmd ==  LIBM_TCMD_DUMP )
	{
		thid_IsDump = sceKernelGetThreadmanIdList( SCE_KERNEL_TMID_Thread, thid_list_first, MAX_NUMBER_OF_THREADS, &thid_count_first ) >= 0;
		
		return thid_IsDump;
	}
	else if( (cmd == LIBM_TCMD_SUSPEND || cmd == LIBM_TCMD_RESUME) && thid_IsDump && cmd != thid_status && !this_IsBusy )
	{
		thid_status = cmd;
		this_IsBusy = true;
		
		int i, j;
		int (*func)(SceUID) =  ( cmd == LIBM_TCMD_SUSPEND ? sceKernelSuspendThread : sceKernelResumeThread );
		SceKernelThreadInfo thinfo;
		
		
		if( cmd == LIBM_TCMD_SUSPEND )
		{
			//~
			
			int Current_id	= sceKernelGetThreadId();
			
			//݂̃XbhXg
			if( sceKernelGetThreadmanIdList( SCE_KERNEL_TMID_Thread, thid_list_Current, MAX_NUMBER_OF_THREADS, &thid_count_Current ) < 0 )
			{
				//݂̃XbhXg擾s
				this_IsBusy = false;
				return 0;
			}
			
			
			//݂̃XbhꗗƁAvOCNɃ_vZ[tXgr
			//Z[tXgƌXbhɈv̂͑ΏۊO
			for( i = 0; i < thid_count_Current; i++ )
			{
				memset(&thinfo, 0, sizeof(SceKernelThreadInfo));
				thinfo.size = sizeof(SceKernelThreadInfo);
				sceKernelReferThreadStatus(thid_list_Current[i], &thinfo);
				
				if
				(
				//	( strcasecmp( "SceImpose",thinfo.name) == 0 )
				//||
					( thinfo.attr & 0x100000 ) == 0x100000
				||
					( thid_list_Current[i] == Current_id )
				)
				{
					thid_list_Current[i] = 0;
					continue;
				}
				
				for( j = 0; j < thid_count_first; j++ )
				{
					if( thid_list_Current[i] == thid_list_first[j] )
					{
						thid_list_Current[i] = 0;
						break;
					}
				}
			}
		}

		//ŏIIȌ݂̃XbhXgɂXbh֒~EĊJ̑
		for( i = 0; i < thid_count_Current; i++ )
		{
			if( thid_list_Current[i] ) ( func )( thid_list_Current[i] );
		}
		
		this_IsBusy = false;

		return 1;
	}
	else if( cmd == LIBM_TCMD_GET )
	{
		return thid_status;
	}
	
	return 0;
}


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//  cusHomeGetStatus
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int cusHomeGetStatus()
{
	return (Context.IsOpen ? 1 : 0);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//  cusHomeClose
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int cusHomeClose()
{
	if(Context.IsOpen)
	{
		CloseMethod();
		setting_flag = false;
		statesflag = false;
		dialog_flag = false;
		Checkflag=0;
		Checktype=0;
		Context.Color.Active_Font = ini.activeColor;
		return 0;
	}
	return -1;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//  Callback_Suspend
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////


int Callback_Suspend(int ev_id, char *ev_name, void *param, int *result)
{
	// Ԃl 0 ł͂Ȃ -1 ԂƁAX[v𖳌ł
	
	// X[v
	if( ev_id == PSP_SYSEVENT_SUSPEND_QUERY)
	{
		cusHomeClose();
	}
	
	return 0;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//  GetNidDisp
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

// ֐
u32 GetSetBaseAddrNidDisp(void)
{
	u32 devkit = sceKernelDevkitVersion();

	if(devkit >= PSP_FIRMWARE(0x500))
	{
		if(devkit >= PSP_FIRMWARE(0x660))//6.60-
		{
			return 0xA3AA8D00;
		}
		else if(devkit >= PSP_FIRMWARE(0x630))//6.30-6.39
		{
			return 0x8A0E1D73;
		}
		else if(devkit >= PSP_FIRMWARE(0x620))//6.20
		{
			return 0x6CDDF704;
		}
		else//5.00-5.50
		{
			return 0xBA1605D2;
		}
	}

	return 0x29B50A82;
}

u32 GetSetBaseAddr(u32 text_addr, u32 text_end)
{
	u32 addr = 0;

	for(; text_addr < text_end; text_addr += 4)
	{
		if(_lw(text_addr) == 0x00000000 && _lw(text_addr + 4) == 0x00112880)
		{
			addr = (text_addr - 0xC);
			break;
		}
	}

	return addr;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//  GetColerNum
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

int GetColerNum(int color)
{
	switch(color){
		case BLACK:			return 1;
		case BLUE:			return 2;
		case RED:			return 3;
		case GREEN:			return 4;
		case DARKBLUE:		return 5;
		case LIGHTBLUE:		return 6;
		case BROWN:			return 7;
		case PINK:			return 8;
		case DARKGREEN:		return 9;
		case YELLOGREEN:	return 10;
		case PURPLE:		return 11;
		case ORANGE:		return 12;
		case LEMON:			return 13;
		case YELLOW:		return 14;
		case SKYBLUE:		return 15;
		case PEARLORANGE:	return 16;
		case GRAY:			return 17;
		case SILVER:		return 18;
		case GOLD:			return 19;
		case WHITE:			return 20;
	}
	return 0;
}


