#ifndef COMMON_H
#define COMMON_H

#include <pspctrl.h>
#include <pspctrl_kernel.h>
#include <pspdisplay.h>
#include <pspdisplay_kernel.h>
#include <pspge.h>
#include <pspimpose_driver.h>
#include <pspkernel.h>
#include <pspmoduleinfo.h>
#include <psppower.h>
#include <psprtc.h>
#include <pspsdk.h>
#include <pspsysclib.h>
#include <pspsysevent.h>
#include <psputilsforkernel.h>
#include <kubridge.h>
#include <systemctrl.h>
#include <systemctrl_se.h>

#include <stdarg.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <locale.h>

#include "cmlib/cmlibaudio.h"
#include "cmlib/cmlibctrl.h"
#include "cmlib/cmlibmenu.h"
#include "pict/pict.h"
#include "color.h"
#include "inilibp.h"
#include "states.h"
#include "getname.h"
#include "sysclib.h"
#include "menu.h"


//}N--------------------------------------------------------

#define MAKE_CALL(a, f)	_sw(0x0C000000 | (((u32)(f) >> 2) & 0x03FFFFFF), a);
#define PSP_FIRMWARE(f) ((((f >> 8) & 0xF) << 24) | (((f >> 4) & 0xF) << 16) | ((f & 0xF) << 8) | 0x10)

#define EBOOTBIN "disc0:/PSP_GAME/SYSDIR/EBOOT.BIN"
#define UMDSFO "disc0:/PSP_GAME/PARAM.SFO"

typedef struct CONFIG {
	char date[128];
	bool setting[10];
	u32 activeColor;
	char week[7][16];
	char month[12][16];
	char lang[18][64];
	char message[6][64];
} CONFIG;


/*
  setting
0	SwapButton
1	Homebrew
2	Mute
3	BatteryWarning
4	StopGame
5	Brightness
6	Description
7	GameName
8	GameID
9	PspStates
10	ChangeColor

*/

/*
  lang
0	Exit Game
1	Reset Game
2	Shutdown
3	Standby
4	Pause
5	Resume
6	Back

7	SwapButton
8	ChangeColor
9	Mute
10	BatteryWarning
11	Stop Game
12	Brightness
13	Description
14	GameName
15	GameID
16	Setting
17	Homebrew
*/
/*
  message
0	Exit Game
1	Reset Game
2	Shutdown
3	Standby
4	Battery is low
5	Are you sure?
*/



typedef struct
{
	int width;
	int height;
	int linewidth;
	int pixelformat;
	u32 *buffer;
} CANVAS;

libm_draw_info dinfo;
libm_vram_info vinfo;

extern CONFIG ini;
extern CANVAS canvas;
extern MenuContext	Context;
extern MenuItem *top,*setting,*stopgame,*pspstates;
extern bool setting_flag, statesflag, dialog_flag;
extern int batterycount;

extern int Checkflag, Checktype;
extern int savetype,ctrltype;
extern int slot_pos, setting_pos;
extern char dialog_str[];

extern char gamename[];
extern int gamelen;
extern char gameid[];

extern bool StopGame;
extern bool maskflag, muteflag;
//extern char SettingLang;
extern char *ColorList[];


//Xbh~
#define	LIBM_TSUSPEND				LIBM_TCMD_SUSPEND
#define	LIBM_TRESUME				LIBM_TCMD_RESUME

#define MAX_NUMBER_OF_THREADS 		64
#define	PSP_SYSEVENT_SUSPEND_QUERY	0x100
#define	ITEM_COUNT					80


void DrowMethod();
void CloseMethod();

void PspLsLibrarySaveStates(char number, int flag);
void PspLsLibraryLoadStates(char number, int flag);

void SaveStates(char number, int flag);
void LoadStates(char number, int flag);

extern PspStates states[];


//*********************************************************
//  String ̕ From 𕶎 To ŒuB
// u̕ String ̃TCY String ̋L̈𒴂ꍇ͖̓`B
//*********************************************************
char *StrReplace( char *String, const char *From, const char *To );
void Redraw();
int GetColerNum(int color);
void SetColorName(int num);
int sceKernelCheckExitCallback(void);


//Xbh~
int libmExecTCmd( int cmd );
int Callback_Suspend(int ev_id, char *ev_name, void *param, int *result);

//ʃtbN
int (*sceDmacplusLcdcSetBaseAddrReal)(void *baseaddr);
u32 GetSetBaseAddr(u32 text_addr, u32 text_end);
u32 GetSetBaseAddrNidDisp(void);
int (*sceDisplaySetFrameBuf_Real)(void *topaddr, int bufferwidth, int pixelformat, int sync);

//LoadExecVSH
int LoadExecVSH(int apitype, char *path);
int (*sceKernelLoadExecVSHEf2)(const char *file, struct SceKernelLoadExecVSHParam *param);
int (*sceKernelLoadExecVSHUmdEmuMs1)(const char *file, struct SceKernelLoadExecVSHParam *param);
int (*sceKernelLoadExecVSHUmdEmuEf1)(const char *file, struct SceKernelLoadExecVSHParam *param);
int (*sceKernelLoadExecVSHMs5)(const char *file, struct SceKernelLoadExecVSHParam *param);
int (*sceKernelLoadExecVSHEf5)(const char *file, struct SceKernelLoadExecVSHParam *param);
u32 GetSetBaseAddrNidLoadExec(int type);


#endif