
/*
 * PSPLINK
 * -----------------------------------------------------------------------
 * Licensed under the BSD license, see LICENSE in PSPLINK root for details.
 *
 * bitmap.c - PSPLINK kernel module bitmap code
 *
 * Copyright (c) 2005 James F <tyranid@gmail.com>
 *
 * $HeadURL: svn://svn.pspdev.org/psp/trunk/psplinkusb/psplink/bitmap.c $
 * $Id: bitmap.c 2026 2006-10-14 13:09:48Z tyranid $
 */

#include <pspkernel.h>
#include <pspdisplay.h>
#include <pspsuspend.h>
#include <pspinit.h>
#include <pspsysmem_kernel.h>

#include <stdio.h>
#include <stdint.h>
#include <string.h>

#include "pict.h"


static int fixed_write(int fd, void *data, int len)
{
	int writelen = 0;
	
	while(writelen < len)
	{
		int ret;
		
		ret = sceIoWrite(fd, data + writelen, len - writelen);
		if(ret <= 0)
		{
			writelen = -1;
			break;
		}
		writelen += ret;
	}
	
	return writelen;
}

int write_8888_data(void *frame, void *pixel_data)
{
	uint8_t *line;
	uint8_t *p;
	int i;
	int h;
	
	line = pixel_data;
	for(h = 271; h >= 0; h--)
	{
		p = frame + (h*512*4);
		for(i = 0; i < 480; i++)
		{
			line[(i*3) + 2] = p[i*4];
			line[(i*3) + 1] = p[(i*4) + 1];
			line[(i*3) + 0] = p[(i*4) + 2];
		}
		line += 480 * 3;
	}
	
	return 0;
}

int write_5551_data(void *frame, void *pixel_data)
{
	uint8_t *line;
	uint16_t *p;
	int i;
	int h;
	
	line = pixel_data;
	for(h = 271; h >= 0; h--)
	{
		p = frame;
		p += (h * 512);
		for(i = 0; i < 480; i++)
		{
			line[(i*3) + 2] = (p[i] & 0x1F) << 3;
			line[(i*3) + 1] = ((p[i] >> 5) & 0x1F) << 3;
			line[(i*3) + 0] = ((p[i] >> 10) & 0x1F) << 3;
		}
		line += 480*3;
	}
	
	return 0;
}

int write_565_data(void *frame, void *pixel_data)
{
	uint8_t *line;
	uint16_t *p;
	int i;
	int h;
	
	line = pixel_data;
	for(h = 271; h >= 0; h--)
	{
		p = frame;
		p += (h * 512);
		for(i = 0; i < 480; i++)
		{
			line[(i*3) + 2] = (p[i] & 0x1F) << 3;
			line[(i*3) + 1] = ((p[i] >> 5) & 0x3F) << 2;
			line[(i*3) + 0] = ((p[i] >> 11) & 0x1F) << 3;
		}
		line += 480*3;
	}
	
	return 0;
}

int write_4444_data(void *frame, void *pixel_data)
{
	uint8_t *line;
	uint16_t *p;
	int i;
	int h;
	
	line = pixel_data;
	for(h = 271; h >= 0; h--)
	{
		p = frame;
		p += (h * 512);
		for(i = 0; i < 480; i++)
		{
			line[(i*3) + 2] = (p[i] & 0xF) << 4;
			line[(i*3) + 1] = ((p[i] >> 4) & 0xF) << 4;
			line[(i*3) + 0] = ((p[i] >> 8) & 0xF) << 4;
		}
		line += 480*3;
	}
	
	return 0;
}

int bitmapWrite(void *frame_addr, void *tmp_buf, int format, const char *file)
{
	BitmapHeader *bmp;
	void *pixel_data = tmp_buf + sizeof(BitmapHeader);
	int fd;
	
	bmp = (BitmapHeader *) tmp_buf;
	memset(bmp, 0, sizeof(BitmapHeader));
	memcpy(bmp->id, BMP_ID, sizeof(bmp->id));
	bmp->filesize = 480*272*3 + sizeof(BitmapHeader);
	bmp->offset = sizeof(BitmapHeader);
	bmp->headsize = 0x28;
	bmp->width = 480;
	bmp->height = 272;
	bmp->planes = 1;
	bmp->bpp = 24;
	bmp->bitmapsize = 480*272*3;
	bmp->hres = 2834;
	bmp->vres = 2834;
	
	switch(format)
	{
		case PSP_DISPLAY_PIXEL_FORMAT_565: write_565_data(frame_addr, pixel_data);
			break;
		case PSP_DISPLAY_PIXEL_FORMAT_5551: write_5551_data(frame_addr, pixel_data); 
			break;
		case PSP_DISPLAY_PIXEL_FORMAT_4444: write_4444_data(frame_addr, pixel_data);
			break;
		case PSP_DISPLAY_PIXEL_FORMAT_8888: write_8888_data(frame_addr, pixel_data);
			break;
	};
	
	if((fd = sceIoOpen(file, PSP_O_WRONLY | PSP_O_CREAT | PSP_O_TRUNC, 0777)) >= 0) {
		fixed_write(fd, tmp_buf, bmp->filesize);
		sceIoClose(fd);
		return 0;
	}
	return -1;
}


//---thanks to prxshot v0.4.0---

#define BMP_SIZE 391734
#define PSP_MEMORY_PARTITION_UMDCACHE 8
#define PSP_MODEL_SLIM 1
#define SHOT_BLK_NAME "shot_blk"


void *allocScreenShotBufferMemory(SceSize size, int *id)
{
    void *mem = NULL;
    int api = sceKernelInitKeyConfig();
    if( /*(game_found || api == PSP_INIT_KEYCONFIG_VSH) &&*/ sceKernelGetModel() >= PSP_MODEL_SLIM ) {
        // use the umd cache only if is a game and slim or superior
        mem = kalloc(size, SHOT_BLK_NAME, id, PSP_MEMORY_PARTITION_UMDCACHE, PSP_SMEM_Low);
    }
    if(!mem) {
        // else get the memory from kernel
        mem = kalloc(size, SHOT_BLK_NAME, id, PSP_MEMORY_PARTITION_KERNEL, PSP_SMEM_Low);
        if(!mem && api == PSP_INIT_KEYCONFIG_GAME) {
            // as a last resort, use the volatile mem
            mem = kalloc_volatile();
        }
    }
	return mem;
}

int takeScreenShot(const char *path)
{
	void *frame_addr;
	SceUID block_id = -1;
	int frame_width, pixel_format;
	unsigned int ptr;
	int ret = -1;

	void *mem = allocScreenShotBufferMemory(BMP_SIZE, &block_id);
	if(mem) {
		sceDisplayWaitVblankStart();
		if(sceDisplayGetFrameBuf(&frame_addr, &frame_width, &pixel_format, PSP_DISPLAY_SETBUF_NEXTFRAME) >= 0 && frame_addr) {
			ptr = (unsigned int)frame_addr;
			ptr |= ptr & 0x80000000 ?  0xA0000000 : 0x40000000;
			bitmapWrite((void *)ptr, mem, pixel_format, path);
			ret = 0;
		}
		block_id >= 0 ? kfree(block_id) : kfree_volatile();
	}
	return ret;
}

/*
int takeScreenShotForBuffer(SSData *data)
{
	void *frame_addr;
	void *buf = NULL;
	SceUID block_id = -1;
	int frame_width, pixel_format;
	unsigned int ptr;
	int ret = -1;

	data->width = SCREEN_SHOT_WIDTH*2;
	data->height = SCREEN_SHOT_HEIGHT*2;
	data->size = data->width*data->height*2;

	memset(data->raw, 0, data->size);

	data->raw = allocScreenShotBufferMemory(data->size, &block_id);
	

	if(data->raw) {
		sceDisplayWaitVblankStart();
		if(sceDisplayGetFrameBuf(&data->size, &frame_width, &pixel_format, PSP_DISPLAY_SETBUF_NEXTFRAME) >= 0 && frame_addr) {
			ptr = (unsigned int)frame_addr;
			ptr |= ptr & 0x80000000 ?  0xA0000000 : 0x40000000;

	//		void *pixel_data = buf + sizeof(BitmapHeader);

			switch(pixel_format)
			{
				case PSP_DISPLAY_PIXEL_FORMAT_565: write_565_data((void *)ptr, data->raw);
					break;
				case PSP_DISPLAY_PIXEL_FORMAT_5551: write_5551_data((void *)ptr, data->raw); 
					break;
				case PSP_DISPLAY_PIXEL_FORMAT_4444: write_4444_data((void *)ptr, data->raw);
					break;
				case PSP_DISPLAY_PIXEL_FORMAT_8888: write_8888_data((void *)ptr, data->raw);
					break;
			};
			read_8888_data(data->raw, data->raw, data->width, data->height);
			ret = 0;
//			memcpy(data->raw, frame_addr, data->size);
		}
		block_id >= 0 ? kfree(block_id) : kfree_volatile();
	}
	return ret;
}
*/